<?php

namespace App\Controllers;

use Dynamite\DatabaseTable as DB;

use Dynamite\Controller;



class ClasesController extends Controller

{

	private $session;

	private $auth;

	private $tablaClases;

	private $tablaMaterias;

	private $tablaUsuarios;
	private $tablaLicenciaturas;

	private $tablaPE;

	private $tablaClaseAlumno;



	function __construct(\Dynamite\Authentication $auth, DB $tablaClases, DB $tablaMaterias, DB $tablaUsuarios, DB $tablaPE, DB $tablaClaseAlumno, DB $tablaLicenciaturas)

	{

		$this->session = new \Dynamite\Session;

		$this->auth = $auth;

		$this->tablaClases = $tablaClases;
		$this->tablaLicenciaturas = $tablaLicenciaturas;

		$this->tablaMaterias = $tablaMaterias;

		$this->tablaUsuarios = $tablaUsuarios;

		$this->tablaPE = $tablaPE;

		$this->tablaClaseAlumno = $tablaClaseAlumno;

	}



	public function ver() {

		$id = $_GET['id'];

		$alumnosClase = $this->tablaClaseAlumno->find('idClase', $id);

		return [

			'template' => 'content/clases/ver',

			'title' => 'Clases - Ver',

			'variables' => [

				'alumnosClase' => $alumnosClase,

				'clase' => $this->tablaClases->findById($id)

			]

		];

	}



	public function calificarShow() {

		return $this->agregar([], true);

	}



	public function calificar() {

		$data = $_POST['claseAlumno'];

		$required = ['idAlumno', 'idClase', 'calificacion', 'tipo'];

		$errors = $this->validate($data, $required);



		if(empty($errors['calificacion']) && ($data['calificacion'] < 0 || $data['calificacion'] > 10)) {

			$errors['calificacion'] = 'La calificación debe de ser un número entre 0.0 y 10.0';

		}

		if (empty($errors['tipo']) && $data['tipo'] != 'ordinario' && $data['tipo'] != 'extraordinario') {

			$errors['tipo'] = 'El tipo de calificación debe de ser ordinario o extraordinario.';
		}

		if(empty($errors)) {

			$clase = $this->tablaClases->findById($data['idClase']);

			$alumno = $this->tablaUsuarios->findById($data['idAlumno']);

			if($clase && $alumno) {

				$claseAlumno = $this->tablaClaseAlumno->findById([

					'idAlumno' => $alumno->id,

					'idClase' => $clase->id

				]);

				if($claseAlumno) {

					$this->session->flash('success', 'Se calificó correctamente al alumno.');

					$this->tablaClaseAlumno->save($data);

					header('location: ./calificar?id='.$data['idClase']);

					return;

				}

				$this->session->flash('info', 'No existe esa relación clase/alumno.');

				return $this->agregar([], true);

			}

			$this->session->flash('error', 'No existe la clase o el alumno que se quiere asignar.');

			return $this->agregar(['old' => $data], true);

		}

		$this->session->flash('error', 'No se pudo asignar el alumno a la clase.');

		return $this->agregar(['errors' => $errors, 'old' => $data], true);

	}



	public function indexCalificaciones() {

		$usuario = $this->auth->getUser();

		if($usuario->esDelTipoUsuario(\App\Entity\Usuario::ADMINISTRADOR)) {
			$clases = $this->tablaClases->find('estado', 'activo');
		} else {
			$clasesget = $this->tablaClases->find('estado', 'activo');
			$clases = array();
			foreach ($clasesget as $clase) {
				if ($clase->idProfesor == $usuario->id) {
					$clases[] = $clase;
				}
			}
		}

		return [

			'template' => 'content/calificaciones/index',

			'title' => 'Calificaciones - Index',

			'variables' => [

				'clases' => $clases,

			]

		];

	}



	public function changeStatus() {

		$id = $_POST['id'] ?? 0;

		$lic = $this->tablaClases->findById($id);

		if($lic){

			if($lic->estado == 'activo') {

				$newStatus = 'inactivo';

			} else {

				$newStatus = 'activo';

			}

			$this->tablaClases->save([

				'id' => $id,

				'estado' => $newStatus

			]);

			$this->session->flash('warning', 'Se cambió el estado de la clase.');

			header('location: ./');

			return;

		}

		$this->session->flash('error', 'No existe la clase a la que se le quiere cambiar el estado.');

		header('location: ./');

		return;

	}



	public function eliminarAsignar() {

		$idAlumno = $_POST['idAlumno'];

		$idClase = $_POST['idClase'];

		$claseAlumno = $this->tablaClaseAlumno->findById([

			'idAlumno' => $idAlumno,

			'idClase' => $idClase

		]);

		if($claseAlumno) {

			$this->tablaClaseAlumno->delete([

				'idAlumno' => $idAlumno,

				'idClase' => $idClase

			]);

			$this->session->flash('warning', 'Se eliminó la asignación de forma correcta.');

			header('location: ./asignar?id='.$idClase);

			return;

		}

		$this->session->flash('error', 'No se pudó eliminar la asignación, ya que no existe.');

		header('location: ./asignar?id='.$idClase);

		return;

	}



	public function asignar() {

		$data = $_POST['claseAlumno'];

		$required = ['idAlumno', 'idClase'];

		$errors = $this->validate($data, $required);

		if(empty($errors)) {

			$clase = $this->tablaClases->findById($data['idClase']);

			$alumno = $this->tablaUsuarios->findById($data['idAlumno']);

			if($clase && $alumno) {

				$claseAlumno = $this->tablaClaseAlumno->findById([

					'idAlumno' => $alumno->id,

					'idClase' => $clase->id

				]);

				if(!$claseAlumno) {

					$this->session->flash('success', 'Se asignó al Alumno correctamente a esta clase.');

					$this->tablaClaseAlumno->save($data);

					header('location: ./asignar?id='.$data['idClase']);

					return;

				}

				$this->session->flash('info', 'Ese Alumno ya esta asignada a la Clase.');

				return $this->agregar();

			}

			$this->session->flash('error', 'No existe la clase o el alumno que se quiere asignar.');

			return $this->agregar(['old' => $data]);

		}

		$this->session->flash('error', 'No se pudo asignar el alumno a la clase.');

		return $this->agregar(['errors' => $errors, 'old' => $data]);

	}



	public function agregar($variables = [], $calificar = false) {

		$clase = $this->tablaClases->findById($_GET['id'] ?? 0);

		if($clase) {

			if(!$calificar) {

				$alumnos = $this->tablaUsuarios->findWhere([

					'estado' => 'activo',

					'tipo_usuario' => \App\Entity\Usuario::ALUMNO

				], 'AND', null, null, null, '=');

			} else {

				$alumnos = $clase->alumnos();

			}

			$alumnosSend = [];

			foreach ($alumnos as $value) {
				if ($calificar || $value->cursarClase($clase->idMateria) && $value->claseLicenciatura($clase->idMateria)) {
					$alumnosSend[] = [

						'value' => $value->id,

						'name' => $value->licenciatura()->nombre . ' | matricula: ' .$value->matricula. ' | nombre: ' . $value->apellido . ' ' . $value->nombre

					];
				}

			}

			$variables['alumnos'] = $alumnosSend;

			$variables['clase'] = $clase;

			$variables['claseAlumno'] = $this->tablaClaseAlumno->find('idClase', $clase->id) ?? [];

			if(!$calificar) {

				return [

					'template' => 'content/clases/asignar',

					'title' => 'Clases - Asignar Alumnos',

					'variables' => $variables

				];

			} else {

				return [

					'template' => 'content/calificaciones/calificar',

					'title' => 'Calificaciones - Calificar Alumnos',

					'variables' => $variables

				];

			}

		}

		$this->session->flash('error', 'La clase con ese ID no existe!');

		header('location: ./');

		return;

	}

	public function getMaterias()
	{
		$idLic = $_POST['id'];
		$lic = $this->tablaLicenciaturas->findByid($idLic);
		$materias = $lic->materias();
		if (isset($_POST['id_clase'])) {
			$clase = $this->tablaClases->findById($_POST['id_clase']);
		}
		$text = '<option value="" selected="">Selecciona una opción</option>';
		foreach ($materias as $materia) {
			if (isset($clase) && $clase->idMateria == $materia->id) {
				$text .= '<option value="'.$materia->id.'" selected>'.$materia->nombre.'</option>';
			} else {
				$text .= '<option value="'.$materia->id.'">'.$materia->nombre.'</option>';
			}
		}
		return [
			'success' => true,
			'print' => $text
		];
	}

	public function index() {

		$usuario = $this->auth->getUser();

		if($usuario->esDelTipoUsuario(\App\Entity\Usuario::ADMINISTRADOR)) {
			$clases = $this->tablaClases->findAll();
		} else {
			$clases = $this->tablaClases->find('idProfesor', $usuario->id);
		}

		return [

			'template' => 'content/clases/index',

			'title' => 'Clases - Index',

			'variables' => [

				'clases' => $clases,

			]

		];

	}



	private function getData() {

		$data = [];

		$profesores = $this->tablaUsuarios->find('tipo_usuario', \App\Entity\Usuario::PROFESOR);

		$data['profesores'] = [];

		foreach ($profesores as $value) {

			$data['profesores'][] = [

				'value' => $value->id,

				'name' => $value->nombre . ' ' . $value->apellido

			];

		}

		$materias = $this->tablaMaterias->find('estado', 'activo');

		$data['materias'] = [];

		foreach ($materias as $value) {

			$data['materias'][] = [

				'value' => $value->id,

				'name' => $value->nombre

			];

		}

		$lics = $this->tablaLicenciaturas->find('estado', 'activo');
		$data['licenciaturas'] = [];

		foreach ($lics as $value) {

			$data['licenciaturas'][] = [

				'value' => $value->id,

				'name' => $value->nombre

			];

		}

		$pe = $this->tablaPE->find('estado', 'activo');

		$data['periodos'] = [];

		foreach ($pe as $value) {

			$data['periodos'][] = [

				'value' => $value->id,

				'name' => $value->nombre

			];

		}

		return $data;

	}



	public function create($variables = []) {

		$data = $this->getData();

		$variables['data'] = $data;
		return [

			'template' => 'content/clases/create',

			'title' => 'Clases - Crear',

			'variables' => $variables

		];

	}



	public function store() {

		$data = $this->storeUpdate();

		if(!is_array($data)) {

			$this->session->flash('info', 'Se creó la clase.');

			header('location: ./');

			return;

		}

		$this->session->flash('error', 'No se pudo registrar la clase.');

		return $this->create(['errors' => $data, 'old' => $_POST['clase'] ?? null]);

	}



	public function edit($variables = []) {

		$data = $this->getData();

		$variables['data'] = $data;

		$id = $_GET['id'];

		$clase = $this->tablaClases->findById($id);

		$variables['clase'] = $clase;

		return [

			'template' => 'content/clases/edit',

			'title' => 'Clases - Editar',

			'variables' => $variables

		];

	}



	public function update() {

		$data = $this->storeUpdate(true);

		if(!is_array($data)) {

			$this->session->flash('info', 'Se actualizó la clase.');

			header('location: ./');

			return;

		}

		$this->session->flash('error', 'No se pudo actualizar la clase.');

		return $this->edit(['errors' => $data, 'old' => $_POST['clase'] ?? null]);

	}



	private function storeUpdate($edit = false) {

		$data = $_POST['clase'];

		$required = ['nombre', 'idMateria', 'idProfesor', 'idPeriodoEscolar'];

		$errors = $this->validate($data, $required);

		if(empty($errors)) {

			if(!$edit){

				$data['fechaRegistro'] = date('Y-m-d H:i:s');

			}

			$this->tablaClases->save($data);

			return true;

		}

		return $errors;

	}

}