<?php

namespace App\Controllers;

use Dynamite\DatabaseTable as DB;

use Dynamite\Controller;



class LicenciaturasController extends Controller

{

	private $session;

	private $tablaLic;

	private $tablaMaterias;

	private $tablaLicMat;



	function __construct(DB $tablaLic, DB $tablaMaterias, DB $tablaLicMat)

	{

		$this->session = new \Dynamite\Session;

		$this->tablaLic = $tablaLic;

		$this->tablaMaterias = $tablaMaterias;

		$this->tablaLicMat = $tablaLicMat;

	}



	public function changeStatus() {

		$id = $_POST['id'] ?? 0;

		$lic = $this->tablaLic->findById($id);

		if($lic){

			if($lic->estado == 'activo') {

				$newStatus = 'inactivo';

			} else {

				$newStatus = 'activo';

			}

			$this->tablaLic->save([

				'id' => $id,

				'estado' => $newStatus

			]);

			$this->session->flash('warning', 'Se cambió el estado de la licenciatura.');

			header('location: ./');

			return;

		}

		$this->session->flash('error', 'No existe la licenciatura a la que se le quiere cambiar el estado.');

		header('location: ./');

		return;

	}



	public function eliminarAsignar() {

		$idMateria = $_POST['idMateria'];

		$idLicenciatura = $_POST['idLicenciatura'];

		$licmat = $this->tablaLicMat->findById([

			'idMateria' => $idMateria,

			'idLicenciatura' => $idLicenciatura

		]);

		if($licmat) {

			$this->tablaLicMat->delete([

				'idMateria' => $idMateria,

				'idLicenciatura' => $idLicenciatura

			]);

			$this->session->flash('warning', 'Se eliminó la asignación de forma correcta.');

			header('location: ./asignar?id='.$idLicenciatura);

			return;

		}

		$this->session->flash('error', 'No se pudó eliminar la asignación, ya que no existe.');

		header('location: ./asignar?id='.$idLicenciatura);

		return;

	}



	public function asignar() {

		$data = $_POST['licmat'];

		$required = ['idLicenciatura', 'idMateria'];

		$errors = $this->validate($data, $required);

		if(empty($errors)) {

			$materia = $this->tablaMaterias->findById($data['idMateria']);

			$licenciatura = $this->tablaLic->findById($data['idLicenciatura']);

			if($materia && $licenciatura) {

				$licmat = $this->tablaLicMat->findById([

					'idMateria' => $materia->id,

					'idLicenciatura' => $licenciatura->id

				]);

				if(!$licmat) {

					$this->session->flash('success', 'Se asignó la Materia a la Licenciatura correctamente.');

					$this->tablaLicMat->save($data);

					header('location: ./asignar?id='.$data['idLicenciatura']);

					return;

				}

				$this->session->flash('info', 'Esa Materia ya esta asignada a la Licenciatura.');

				return $this->agregar();

			}

			$this->session->flash('error', 'No existe la materia/licenciatura que se quiere asignar.');

			return $this->agregar(['old' => $data]);

		}

		$this->session->flash('error', 'No se pudo asignar la materia.');

		return $this->agregar(['errors' => $errors, 'old' => $data]);

	}



	public function agregar($variables = []) {

		$licenciatura = $this->tablaLic->findById($_GET['id'] ?? 0);

		if($licenciatura) {

			$materias = $this->tablaMaterias->find('estado', 'activo');

			$materiasSend = [];

			foreach ($materias as $value) {

				$materiasSend[] = [

					'value' => $value->id,

					'name' => $value->nombre

				];

			}

			$variables['materias'] = $materiasSend;

			$variables['licenciatura'] = $licenciatura;

			$variables['licmat'] = $this->tablaLicMat->find('idLicenciatura', $licenciatura->id) ?? [];

			return [

				'template' => 'content/licenciaturas/materias',

				'title' => 'Licenciaturas - Asignar Materias',

				'variables' => $variables

			];

		}

		$this->session->flash('error', 'La licenciatura con ese ID no existe!');

		header('location: ./');

		return;

	}



	public function index() {

		$licenciaturas = $this->tablaLic->findAll();
		return [

			'template' => 'content/licenciaturas/index',

			'title' => 'Licenciaturas - Inicio',

			'variables' => [
				'licenciaturas' => $licenciaturas,
			]

		];

	}



	public function create($variables = []) {

		return [

			'template' => 'content/licenciaturas/create',

			'title' => 'Licenciaturas - Crear',

			'variables' => $variables

		];

	}



	public function store() {

		$data = $this->storeUpdate();

		if(!is_array($data)) {

			$this->session->flash('info', 'Se creó la licenciatura.');

			header('location: ./');

			return;

		}

		$this->session->flash('error', 'No se pudo subir el registro.');

		return $this->create(['errors' => $data, 'old' => $_POST['lic']]);

	}



	public function edit($variables = []) {

		$id = $_GET['id'];

		$lic = $this->tablaLic->findById($id);

		$variables['licenciatura'] = $lic;

		return [

			'template' => 'content/licenciaturas/edit',

			'title' => 'Licenciaturas - Editar',

			'variables' => $variables

		];

	}



	public function update() {

		$data = $this->storeUpdate(true);

		if(!is_array($data)) {

			$this->session->flash('info', 'Se actualizó la licenciatura.');

			header('location: ./');

			return;

		}

		$this->session->flash('error', 'No se pudo actualizar el registro.');

		return $this->edit(['errors' => $data, 'old' => $_POST['lic']]);

	}



	public function storeUpdate($edit = false) {

		$data = $_POST['lic'];

		$required = ['nombre'];

		$errors = $this->validate($data, $required);

		if(empty($errors)) {

			if(!$edit){

				$data['fechaRegistro'] = date('Y-m-d H:i:s');

			}

			$this->tablaLic->save($data);

			return true;

		}

		return $errors;

	}

}