<?php

namespace App\Controllers;

use Dynamite\DatabaseTable as DB;

use Dynamite\Controller;



class UsuarioController extends Controller

{

	private $session;

	private $tablaUsuarios;

	private $tablaPagos;

	private $tablaLicenciaturas;
	private $auth;
	private $tablaTitulacion;
	private $tablaMensajes;



	function __construct(DB $tablaUsuarios, DB $tablaPagos, DB $tablaLicenciaturas, DB $tablaTitulacion, DB $tablaMensajes, \Dynamite\Authentication $auth)

	{

		$this->session = new \Dynamite\Session;

		$this->tablaUsuarios = $tablaUsuarios;

		$this->auth = $auth;
		$this->tablaTitulacion = $tablaTitulacion;
		$this->tablaPagos = $tablaPagos;

		$this->tablaLicenciaturas = $tablaLicenciaturas;
		$this->tablaMensajes = $tablaMensajes;

	}


	public function exportExcel() {

		$id = $_GET['id'] ?? 0;

		$usuario = $this->tablaUsuarios->findById($id);

		if($usuario) {

			$pagos = $this->tablaPagos->find('idAlumno', $usuario->id, 'fecha DESC');

			$filename = 'Pagos_del_alumno_' .$usuario->id .'.xls';

			$pagoUsuario = true;

		} else {

			$pagos = $this->tablaPagos->findAll('fecha DESC');

			$filename = 'Pagos.xls';

			$pagoUsuario = false;

		}

		header('Content-Type: application/vnd.ms-excel');

		header('Content-Disposition: attachment; filename='.$filename);

		if($pagoUsuario) {

			$title = ['Pagos del alumno:', $usuario->nombre.' '.$usuario->apellido, 'Matricula: ', $usuario->matricula];

			$arrayValues = ['Fecha de Pago', 'Monto'];

		} else {

			$title = ['Reporte de todos los pagos'];

			$arrayValues = ['Fecha de pago', 'Monto', 'Alumno', 'Matricula'];

		}

		if(count($pagos) > 0) {

			echo implode("\t", array_values($title)) . "\n";

			echo implode("\t", array_values($arrayValues)) . "\n";

			foreach($pagos as $pago) {

				if($pagoUsuario) {

					$values = [

						date('m-d-Y', strtotime($pago->fecha)),

						$pago->monto

					];

				} else {

					$alumno = $pago->alumno();

					$values = [

						date('m-d-Y', strtotime($pago->fecha)),

						$pago->monto,

						$alumno->nombre,

						$alumno->matricula,

					];

				}

				echo implode("\t", array_values($values)) . "\n";

			}

		} else {

			echo 'No hay datos a exportar';

		}

		exit();

	}



	public function pagos($variables = []) {

		$id = $_GET['id'] ?? 0;

		$usuario = $this->tablaUsuarios->findById($id);

		if($usuario) {

			if($usuario->esDelTipoUsuario(\App\Entity\Usuario::ALUMNO)) {

				$variables['alumno'] = $usuario;

				$pagos = $this->tablaPagos->find('idAlumno', $usuario->id);

				$variables['pagos'] = $pagos;

				return [

					'template' => 'content/usuarios/pagar',

					'title' => 'Alumnos - Pagar',

					'variables' => $variables

				];

			}

			$this->session->flash('error', 'Ese usuario no es un alumno.');

			header('location: ./');

			return;

		}

		$this->session->flash('error', 'Ese usuario no existe.');

		header('location: ./');

		return;

	}



	public function borrarPago() {

		$id = $_POST['id'];

		$idAlumno = $_POST['idAlumno'];

		$pago = $this->tablaPagos->findById($id);

		if($pago) {

			$this->tablaPagos->delete($pago->id);

			$this->session->flash('info', 'Se eliminó el pago.');

			header('location: ./pagos?id='.$idAlumno);

			return;

		}

		$this->session->flash('error', 'No existe el pago.');

		header('location: ./pagos?id='.$idAlumno);

		return;

	}



	public function subirPago() {

		$data = $_POST['pago'];

		$required = ['idAlumno', 'monto', 'fecha'];

		$errors = $this->validate($data, $required);

		if(empty($errors['monto']) && !is_numeric($data['monto'])) {

			$errors['monto'] = 'El monto debe de ser un número';

		}

		if(empty($errors)) {

			$alumno = $this->tablaUsuarios->findById($data['idAlumno']);

			if($alumno) {

				$this->tablaPagos->save($data);

				$this->session->flash('info', 'Se subió el pago correctamente');

				header('location: ./pagos?id='.$data['idAlumno']);

				return;

			}

			$this->session->flash('error', 'No existe el alumno.');

			return $this->pagos(['old' => $data]);

		}

		$this->session->flash('error', 'No se pudó subir el pago.');

		return $this->pagos(['errors' => $errors, 'old' => $data]);

	}



	public function changeStatusProfesor() {

		return $this->changeStatus(\App\Entity\Usuario::PROFESOR);

	}



	public function changeStatusAlumno() {

		return $this->changeStatus(\App\Entity\Usuario::ALUMNO);

	}

	public function changeServicio()
	{
		$id = $_POST['id'] ?? 0;
		$user = $this->tablaUsuarios->findById($id);
		if($user){
			if($user->servicio == 'no') {
				$newStatus = 'si';
			} else {
				$newStatus = 'no';
			}
			$this->tablaUsuarios->save([
				'id' => $id,
				'servicio' => $newStatus
			]);
			$this->session->flash('warning', 'Se cambió el servicio del alumno '.$user->nombre.'.');
			header('location: ./');
			return;
		}
		$this->session->flash('error', 'No existe el alumno a el que se le quiere cambiar el servicio.');
		header('location: ./');
		return;
	}



	private function changeStatus($tipo) {
		$id = $_POST['id'] ?? 0;
		$lic = $this->tablaUsuarios->findById($id);
		if($lic){
			if($lic->estado == 'activo') {
				$newStatus = 'inactivo';
			} else {
				$newStatus = 'activo';
			}
			$this->tablaUsuarios->save([
				'id' => $id,
				'estado' => $newStatus
			]);
			$this->session->flash('warning', 'Se cambió el estado del '.$tipo.'.');
			header('location: ./');
			return;
		}
		$this->session->flash('error', 'No existe el '.$tipo.' a el que se le quiere cambiar el estado.');
		header('location: ./');
		return;
	}



	public function index($tipo) {

		$usuarios = $this->tablaUsuarios->find('tipo_usuario', $tipo);

		$variables = [];

		$variables['usuarios'] = $usuarios;

		$variables['tipoUsuario'] = $tipo;

		return $variables;

		

	}



	public function indexProfesor() {

		$variables = $this->index(\App\Entity\Usuario::PROFESOR);

		$variables['sectionTitle'] = 'Profesores';

		return [

			'template' => 'content/usuarios/index',

			'title' => 'Profesores - Index',

			'variables' => $variables

		];

	}



	public function indexAlumno() {

		$variables = $this->index(\App\Entity\Usuario::ALUMNO);

		$variables['sectionTitle'] = 'Alumnos';

		return [

			'template' => 'content/usuarios/index',

			'title' => 'Alumnos - Index',

			'variables' => $variables

		];

	}



	public function createProfesor($variables = []) {

		$variables['isProfesor'] = true;

		$variables['sectionTitle'] = 'Crear Profesor';

		return $this->create($variables);

	}



	public function createAlumno($variables = []) {

		$variables['sectionTitle'] = 'Crear Alumno';

		$lics = $this->tablaLicenciaturas->find('estado', 'activo');

		$licenciaturas = [];

		foreach ($lics as $key => $lic) {

			$licenciaturas[$key]['value'] = $lic->id;

			$licenciaturas[$key]['name'] = $lic->nombre;

		}

		$variables['licenciaturas'] = $licenciaturas;

		return $this->create($variables, true);

	}



	public function create($variables = [], $alumno = false) {
		$title = $alumno ? 'Alumnos - Crear' : 'Profesores - Crear';
		return [

			'template' => 'content/usuarios/create',

			'title' => $title,

			'variables' => $variables

		];

	}



	public function storeAlumno() {

		$data = $this->store(\App\Entity\Usuario::ALUMNO);

		if(!is_array($data)) {

			$this->session->flash('info', 'Se creó el alumno.');

			header('location: ./');

			return;

		}

		$this->session->flash('error', 'No se pudó dar de alta al alumno.');

		return $this->createAlumno(['errors' => $data, 'old' => $_POST['usuario']]);

	}



	public function storeProfesor() {

		$data = $this->store(\App\Entity\Usuario::PROFESOR);

		if(!is_array($data)) {

			$this->session->flash('info', 'Se creó al profesor.');

			header('location: ./');

			return;

		}

		$this->session->flash('error', 'No se pudó dar de alta al profesor.');

		return $this->createProfesor(['errors' => $data, 'old' => $_POST['usuario']]);

	}



	public function store($tipo = \App\Entity\Usuario::ALUMNO) {

		$data = $this->storeUpdate(false, $tipo);

		if(!is_array($data)) {

			return true;

		}

		return $data;

	}



	public function editAlumno($variables = []) {

		$variables['sectionTitle'] = 'Editar Alumno';

		$lics = $this->tablaLicenciaturas->find('estado', 'activo');

		$licenciaturas = [];

		foreach ($lics as $key => $lic) {

			$licenciaturas[$key]['value'] = $lic->id;

			$licenciaturas[$key]['name'] = $lic->nombre;

		}

		$variables['licenciaturas'] = $licenciaturas;

		return $this->edit($variables);

	}



	public function editProfesor($variables = []) {

		$variables['isProfesor'] = true;

		$variables['sectionTitle'] = 'Editar Profesor';

		return $this->edit($variables);

	}



	public function edit($variables = []) {

		$id = $_GET['id'];

		$usuario = $this->tablaUsuarios->findById($id);

		$variables['usuario'] = $usuario;

		return [

			'template' => 'content/usuarios/edit',

			'title' => 'Editar Usuario',

			'variables' => $variables

		];

	}


	public function updateProfesor() {

		return $this->update(\App\Entity\Usuario::PROFESOR);

	}



	public function updateAlumno() {

		return $this->update(\App\Entity\Usuario::ALUMNO);

	}



	public function update($tipo) {

		$data = $this->storeUpdate(true, $tipo);

		if(!is_array($data)) {

			$this->session->flash('info', 'Se actualizó al '.ucfirst($tipo).'.');

			header('location: ./');

			return;

		}

		$this->session->flash('error', 'No se pudo actualizar al '.ucfirst($tipo).'.');

		$lics = $this->tablaLicenciaturas->find('estado', 'activo');

		$licenciaturas = [];

		foreach ($lics as $key => $lic) {

			$licenciaturas[$key]['value'] = $lic->id;

			$licenciaturas[$key]['name'] = $lic->nombre;

		}

		return $this->edit(['licenciaturas' => $licenciaturas, 'errors' => $data, 'old' => $_POST['usuario']]);

		

	}



	public function storeUpdate($edit = false, $tipo = null) {

		$data = $_POST['usuario'];

		$required = ['nombre', 'apellido', 'correo', 'usuario'];

		if(!$edit){

			$required[] = 'password';

			$required[] = 'password_confirm';

		}

		if($tipo == \App\Entity\Usuario::ALUMNO) { //PROFESOR

			$required[] = 'matricula';

			$required[] = 'modalidad';

			// $required[] = 'referenciaBancaria';

			$required[] = 'idLicenciatura';
			$required[] = 'estado';
			// $required[] = 'curp';
			// $required[] = 'calle';
			// $required[] = 'no_exterior';
			// $required[] = 'colonia';
			// $required[] = 'cp';
			// $required[] = 'pais';
			// $required[] = 'dom_estado';

		} else {
			$required[] = 'telefono';

		}

		$errors = $this->validate($data, $required);

		if (empty($errors['correo']) && filter_var($data['correo'], FILTER_VALIDATE_EMAIL) == false) {

			$errors['correo'] = 'El email ingresado no es valido';

		}

		else{

			$data['correo'] = strtolower($data['correo']);

			if(count($this->tablaUsuarios->find('correo', $data['correo'])) > 0) {

				if($edit) {

					$usuario = $this->tablaUsuarios->find('correo', $data['correo'])[0];

					if($usuario->id != $data['id']) {

						$errors['correo'] = 'Ya hay un registro con ese correo';

					}

				} else {

					$errors['correo'] = 'Ya hay un registro con ese correo';

				}

			}

		}

		if(empty($errors['usuario']) && count($this->tablaUsuarios->find('usuario', $data['usuario'])) > 0) {

			$data['usuario'] = strtolower($data['usuario']);

			if(count($this->tablaUsuarios->find('usuario', $data['usuario'])) > 0) {

				if($edit) {

					$usuario = $this->tablaUsuarios->find('usuario', $data['usuario'])[0];

					if($usuario->id != $data['id']) {

						$errors['usuario'] = 'Ya hay un registro con ese usuario';

					}

				} else {

					$errors['usuario'] = 'Ya hay un registro con ese usuario';

				}

			}

		}

		if($tipo == \App\Entity\Usuario::ALUMNO) {

			$data['matricula'] = strtoupper($data['matricula']);

			if(count($this->tablaUsuarios->find('matricula', $data['matricula'])) > 0) {

				if($edit) {

					$usuario = $this->tablaUsuarios->find('matricula', $data['matricula'])[0];

					if($usuario->id != $data['id']) {

						$errors['matricula'] = 'Ya hay un registro con esa matricula';

					}

				} else {

					$errors['matricula'] = 'Ya hay un registro con esa matricula';

				}

			}

		}

		if(!$edit && empty($errors['password']) && $data['password'] != $data['password_confirm']) {

			$errors['password_confirm'] = 'Las contraseñas no coinciden';

		}

		if($edit && empty($errors['password']) && $data['password'] != $data['password_confirm']) {

		$errors['password_confirm'] = 'Las contraseñas no coinciden '.ucfirst($tipo).'.';

		}

		if(empty($errors)) {

			if(isset($data['tipo_usuario'])) {

				unset($data['tipo_usuario']);

			}

			if(!$edit){

				$data['fechaRegistro'] = date('Y-m-d H:i:s');

				$data['password'] = password_hash($data['password'], PASSWORD_DEFAULT);

				$data['tipo_usuario'] = $tipo;

			}

			if($edit){
				
				$data['tipo_usuario'] = $tipo;
				if ($data['password'] != NULL){

					//$data['fechaRegistro'] = date('Y-m-d H:i:s');

					$data['password'] = password_hash($data['password'], PASSWORD_DEFAULT);
				}
				else {
					unset($data['password']);
				}

			}

			unset($data['password_confirm']);

			$usuario = $this->tablaUsuarios->save($data);
			if ($usuario && $usuario->tipo_usuario == 'alumno') {
				$urlData = [];
				$urls = false;
				foreach ($_FILES['usuario']['name'] as $fileName => $value) {
					if ($value != '') {
						$tmp = $_FILES['usuario']['tmp_name'][$fileName];
						$pathSend = 'public/files/alumnos/'.$usuario->id;
						$pathBack = 'files/alumnos/'.$usuario->id;
						$dataSubida = $this->subirArchivo($tmp, $value, $pathSend, $fileName, $pathBack, ['jpg', 'jpeg', 'pdf', 'png']);
						if (!isset($dataSubida['error']) && isset($dataSubida['success'])) {
							$urlData[$fileName] = $dataSubida['success'];
						}
						$urls = true;
					}
				}
				if ($urls) {
					$urlData['id'] = $usuario->id;
					$this->tablaUsuarios->save($urlData, false);
				}
			}

			return true;

		}

		return $errors;

	}

	public function subirDocumentos() {
		$usuario = $this->auth->getUser();
		$urlData = [];
		$urls = false;
		foreach ($_FILES['usuario']['name'] as $fileName => $value) {
			if ($value != '') {
				$tmp = $_FILES['usuario']['tmp_name'][$fileName];
				$pathSend = 'public/files/alumnos/'.$usuario->id;
				$pathBack = 'files/alumnos/'.$usuario->id;
				$dataSubida = $this->subirArchivo($tmp, $value, $pathSend, $fileName, $pathBack, ['jpg', 'jpeg', 'pdf', 'png']);
				if (!isset($dataSubida['error']) && isset($dataSubida['success'])) {
					$urlData[$fileName] = $dataSubida['success'];
				}
				$urls = true;
			}
		}
		if ($urls) {
			$urlData['id'] = $usuario->id;
			$this->tablaUsuarios->save($urlData, false);
		}
		return $this->documentos();
	}

	public function documentos()
	{
		$id = $_GET['id'];
		$user = $this->auth->getUser();
		$alumno = $this->tablaUsuarios->findById($id);
		if ($user->tipo_usuario == 'alumno' && $alumno->id != $user->id) {
			$this->session->flash('error', 'No tienes permiso para ver esos documentos');
			header('location: ./');
			return;
		} else {
			return [
				'template' => 'content/alumno/documentos',
				'title' => 'Documentos',
				'variables' => [
					'alumno' => $alumno,
				]
			];
		}
	}

	public function servicio()
	{
		$id = $_GET['id'];
		$user = $this->auth->getUser();
		$alumno = $this->tablaUsuarios->findById($id);
		if ($user->tipo_usuario == 'alumno' && $alumno->id != $user->id) {
			$this->session->flash('error', 'No tienes permiso para ver esta sección.');
			header('location: ./');
			return;
		} else {
			return [
				'template' => 'content/alumno/servicio',
				'title' => 'Servicio Social',
				'variables' => [
					'alumno' => $alumno,
				]
			];
		}
	}

	public function mensajesAlumno()
	{
		$user = $this->auth->getUser();
		return [
			'template' => 'content/usuarios/mensajesMios',
			'title' => 'Mis Mensajes',
			'variables' => [
				'mensajes' => $user->getMessages(10)
			]
		];
	}

	public function mensajeria($variables = [])
	{
		$id = $_GET['id'];
		$user = $this->tablaUsuarios->findById($id);
		if ($user) {
			return [
				'template' => 'content/usuarios/mensajes',
				'title' => 'Mensajes',
				'variables' => $variables
			];
		}
		header('location: ./alumnos/');
		return;
	}

	public function enviarMensaje()
	{
		$datos = $_POST['mensaje'];
		$id = $datos['id'];
		$mensaje = $datos['mensaje'];
		$user = $this->auth->getUser();
		$userReceiver = $this->tablaUsuarios->findById($id);
		if ($userReceiver) {
			$this->tablaMensajes->save([
				'mensaje' => $mensaje,
				'id_usuario_recibe' => $id,
				'id_usuario_envio' => $user->id,
				'fecha' => date('Y-m-d H:i:s')
			]);
			return $this->mensajeria(['alert' => 'Mensaje enviado.']);
		}
		header('location: ./alumnos/');
		return;
	}

	public function titulacionData($variables = [])
	{
		if (count($variables) > 0) {
			$id = $variables['old']['id_usuario'];
		} else {
			$id = $_GET['id'];
		}
		$user = $this->tablaUsuarios->findById($id);
		if ($user) {
			$tit = $this->tablaTitulacion->findById($user->id) ?? null;
			if ($tit) {
				$tit->fecha_fotografias = date('Y-m-d', strtotime($tit->fecha_fotografias));
				$tit->fecha_acta = date('Y-m-d', strtotime($tit->fecha_acta));
				$tit->fecha_sicyt = date('Y-m-d', strtotime($tit->fecha_sicyt));
				$tit->liberacion = date('Y-m-d', strtotime($tit->liberacion));
			}
			$variables['titulacion'] = $tit;
			return [
				'template' => 'content/titulacion/index',
				'title' => 'Fechas Titulación',
				'variables' => $variables
			];
		}
		header('location: ./alumnos/');
		return;
	}

	public function titulacion()
	{
		$id = $_GET['id'];
		$user = $this->auth->getUser();
		$alumno = $this->tablaUsuarios->findById($id);
		if ($user->tipo_usuario == 'alumno' && $alumno->id != $user->id) {
			$this->session->flash('error', 'No tienes permiso para ver esta sección.');
			header('location: ./');
			return true;
		} else {
			$fechas = $this->tablaTitulacion->findById($user->id) ?? null;
			if ($fechas) {
				$today = date('Y-m-d');
				$todayLate = strtotime($today) + 24*59*60;
				if ($fechas->liberacion != '0000-00-00 00:00:00') {
					$porcentaje = '100';
					$text = '4/4';
				} else if ($fechas->fecha_sicyt != '0000-00-00 00:00:00') {
					$porcentaje = '75';
					$text = '3/4';
				} else if ($fechas->fecha_acta != '0000-00-00 00:00:00') {
					$porcentaje = '50';
					$text = '2/4';
				} else if ($fechas->fecha_fotografias != '0000-00-00 00:00:00') {
					$porcentaje = '25';
					$text = '1/4';
				} else {
					$porcentaje = '0';
					$text = '0/4';
				}
			} else {
				$porcentaje = '0';
				$text = 'Aún no hay una fecha establecida.';
			}
			return [
				'template' => 'content/alumno/titulacion',
				'title' => 'Titulación',
				'variables' => [
					'alumno' => $alumno,
					'fechas' => $fechas ?? null,
					'porcentaje' => $porcentaje,
					'text' => $text
				]
			];
		}
	}

	public function cambiarFechas()
	{
		$data = $_POST['titulacion'];
		$errors = [];
		$user = $this->tablaUsuarios->findById($data['id_usuario']);
		if ($user) {
			if (empty($errors)) {
				$this->tablaTitulacion->save($data);
				header('location: ./fechas-titulacion?id=6');
				return;
			}
		}
		return $this->titulacionData([
			'errors' => $errors,
			'old' => $data
		]);
	}

	protected function subirArchivo($file, $filename, $path, $newName, $pathSave, $validExtensions = [])
	{
		$ext = pathinfo($filename, PATHINFO_EXTENSION );
		$pathNew = __DIR__.'/../../'.$path;
		if (empty($validExtensions) || in_array($ext, $validExtensions)) {
			if (! file_exists($pathNew)) {
				mkdir($pathNew);
			}
			$fileTotalName = $pathNew.'/'.$newName.'.'.$ext;
			if (move_uploaded_file($file, $fileTotalName)) {
				return [
					'success' => $pathSave.'/'.$newName.'.'.$ext
				];
			} else {
				return [
					'error' => 'No se pudo subir el archivo.'
				];
			}
		} else {
			return [
				'error' => 'La extensión es invalida.'
			];
		}
	}

}