<?php

namespace Dynamite;



class DatabaseTable {

	private $pdo;

	private $table;

	private $primaryKeys;

	private $className;

	private $constructorArgs;



	public function __construct(\PDO $pdo, string $table, array $primaryKeys, string $className = '\stdClass', array $constructorArgs = []) {

		$this->pdo = $pdo;

		$this->table = $table;

		$this->primaryKeys = $primaryKeys;

		$this->className = $className;

		$this->constructorArgs = $constructorArgs;

	}



	private function query($sql, $parameters = []) {
		$query = $this->pdo->prepare($sql);

		$query->execute($parameters);

		return $query;

	}	



	public function total($field = null, $value = null) {

		$sql = 'SELECT COUNT(*) FROM `' . $this->table . '`';

		$parameters = [];



		if (!empty($field)) {

			$sql .= ' WHERE `' . $field . '` = :value';

			$parameters = ['value' => $value];

		}

		

		$query = $this->query($sql, $parameters);

		$row = $query->fetch();

		return $row[0];

	}



	public function findById($value) {

		$query = 'SELECT * FROM `' . $this->table . '` WHERE `';



		$parameters = [];



		if(is_array($value)) {

			$counter = 0;

			foreach ($this->primaryKeys as $key => $primaryKey) {

				$query .= $primaryKey . '` = :' . $key;

				if($counter < count($this->primaryKeys) - 1) {

					$query .= ' AND `';

				}

				$parameters[$key] = $value[$key];

				$counter++;

			}

		} else {

			$query .= $this->primaryKeys[0] . '` = :value';

			$parameters['value'] = $value;

		}



		$query = $this->query($query, $parameters);



		return $query->fetchObject($this->className, $this->constructorArgs);

	}



	public function find($column, $value, $orderBy = null, $limit = null, $offset = null, $delimiter = '=') {

		$query = 'SELECT * FROM ' . $this->table . ' WHERE ' . $column . ' '.$delimiter.' :value';



		$parameters = [

			'value' => $value

		];



		if ($orderBy != null) {

			$query .= ' ORDER BY ' . $orderBy;

		}



		if ($limit != null) {

			$query .= ' LIMIT ' . $limit;

		}



		if ($offset != null) {

			$query .= ' OFFSET ' . $offset;

		}



		$query = $this->query($query, $parameters);



		return $query->fetchAll(\PDO::FETCH_CLASS, $this->className, $this->constructorArgs);

	}



	public function findWhere($values, $orAnd = 'OR', $orderBy = null, $limit = null, $offset = null, $delimiter = 'LIKE', $together = []) {

		$query = 'SELECT * FROM ' . $this->table . ' WHERE ';



		$count = 0;

		$isTogether = false;

		$togetherIndex = -1;



		foreach ($values as $column => $value) {

			if($isTogether == false && isset($together[$count])) {

				$query .= '(';

				$isTogether = true;

				$togetherIndex = $count;

			}

			$query .= $column . ' ';

			if(is_array($delimiter)) {

				$query .= $delimiter[$column] ?? 'LIKE';

			} else {

				$query .= $delimiter;

			}

			$query .= ' :' . $column;

			if($isTogether) {

				if($count != $togetherIndex && isset($together[$count])) {

					$query .= ')';

				}

			}

			if($count < count($values) - 1) {

				if(is_array($orAnd)) {

					if(isset($orAnd[$column])) {

						$query .= ' '.$orAnd[$column].' ';

					} else {

						$query .= ' OR ';

					}

				} else {

					$query .= ' '.$orAnd.' ';

				}

			}

			$count++;

		}



		if ($orderBy != null) {

			$query .= ' ORDER BY ' . $orderBy;

		}



		if ($limit != null) {

			$query .= ' LIMIT ' . $limit;

		}



		if ($offset != null) {

			$query .= ' OFFSET ' . $offset;

		}


		$query = $this->query($query, $values);



		return $query->fetchAll(\PDO::FETCH_CLASS, $this->className, $this->constructorArgs);

	}



	private function insert($fields) {

		$query = 'INSERT INTO `' . $this->table . '` (';



		foreach ($fields as $key => $value) {

			$query .= '`' . $key . '`,';

		}



		$query = rtrim($query, ',');



		$query .= ') VALUES (';





		foreach ($fields as $key => $value) {

			$query .= ':' . $key . ',';

		}



		$query = rtrim($query, ',');



		$query .= ')';



		$fields = $this->processDates($fields);



		$this->query($query, $fields);



		if(count($this->primaryKeys) == 1){

			return $this->pdo->lastInsertId();

		}



		$returnArray = [];

		foreach ($this->primaryKeys as $key => $primaryKey) {

			$returnArray[$primaryKey] = $fields[$primaryKey];

		}

		return $returnArray;



	}





	private function update($fields) {

		$query = ' UPDATE `' . $this->table .'` SET ';



		foreach ($fields as $key => $value) {

			$query .= '`' . $key . '` = :' . $key . ',';

		}



		$query = rtrim($query, ',');



		$query .= ' WHERE `';



		$counter = 0;

		if(count($this->primaryKeys) != 1) {

			foreach ($this->primaryKeys as $key => $primaryKey) {

				$query .= $primaryKey . '` = :' . $key;

				if($counter < count($this->primaryKeys) - 1) {

					$query .= ' AND `';

				}

				$counter++;

				$fields[$key] = $fields[$primaryKey];

			}

		} else {

			$query .= $this->primaryKeys[0] .'` = :primaryKey';

			$fields['primaryKey'] = $fields[$this->primaryKeys[0]];

		}



		$fields = $this->processDates($fields);



		$this->query($query, $fields);

	}





	public function delete($values) {

		$query = 'DELETE FROM `' . $this->table . '` WHERE `';

		$parameters = []; 

		if(is_array($values)) {

			$counter = 0;

			foreach ($values as $key => $value) {

				$parameters[$key] = $value;

				$query .= $key. '` = :' . $key;

				if($counter < count($values) - 1) {

					$query .= ' AND `';

				}

				$counter++;

			}

		} else {

			$query .= $this->primaryKeys[0] . '` = :id';

			$parameters['id'] = $values;

		}



		$this->query($query, $parameters);

	}



	public function deleteWhere($column, $value) {

		$query = 'DELETE FROM ' . $this->table . ' WHERE ' . $column . ' = :value';



		$parameters = [

			'value' => $value

		];



		$query = $this->query($query, $parameters);

	}



	public function findAll($orderBy = null, $limit = null, $offset = null) {

		$query = 'SELECT * FROM ' . $this->table;



		if ($orderBy != null) {

			$query .= ' ORDER BY ' . $orderBy;

		}



		if ($limit != null) {

			$query .= ' LIMIT ' . $limit;

		}



		if ($offset != null) {

			$query .= ' OFFSET ' . $offset;

		}



		$result = $this->query($query);



		return $result->fetchAll(\PDO::FETCH_CLASS, $this->className, $this->constructorArgs);

	}



	private function processDates($fields) {

		foreach ($fields as $key => $value) {

			if ($value instanceof \DateTime) {

				$fields[$key] = $value->format('Y-m-d');

			}

		}



		return $fields;

	}





	public function save($record) {

		$entity = new $this->className(...$this->constructorArgs);



		try {

			foreach ($this->primaryKeys as $key => $primaryKey) {

				if (isset($record[$key]) && $record[$key] == '') {

					$record[$key] = null;

				}

			}

			$insertId = $this->insert($record);

			if(!is_array($insertId)){

				$entity->{$this->primaryKeys[0]} = $insertId;

			} else {

				foreach ($insertId as $key => $value) {

					$entity->$key = $value;

				}

			}



		}

		catch (\PDOException $e) {

			try {

				$this->update($record);

			} catch(\PDOException $ee) {

				print_r($e);

			}

		}



		foreach ($record as $key => $value) {

			if (!empty($value)) {

				$entity->$key = $value;	

			}			

		}



		return $entity;	

	}

}