<?php

namespace Dynamite;



class EntryPoint {

	private $route;

	private $method;

	private $routes;

	private $config;

	private $session;

	private $helper;

	private $auth;



	public function __construct(string $route, string $method, \Dynamite\Routes $routes) {

		$this->route = $route;

		$this->routes = $routes;

		$this->method = $method;

		$this->config = new \Config\Configuration();

		$this->session = new \Dynamite\Session();

		$this->checkUrl();

	}



	private function checkUrl() {

		if ($this->route !== strtolower($this->route)) {

			http_response_code(301);

			header('location: ' . $this->config->routeRelative . strtolower($this->route));

			return;

		}

	}



	private function loadTemplate($templateFileName, $variables = []) {

		return $this->helper->include($templateFileName, $variables);

	}



	public function run() {



		$routes = $this->routes->getRoutes();



		$this->auth = $this->routes->getAuthentication();



		$thePermissions = false;

		$pathArray = $this->nestRoute($this->route, $routes);

		if(is_array($pathArray)){

			foreach ($pathArray as $value) {

				$routes = $routes[$value];

				if(isset($routes['login']) && !is_array($routes['login'])) {

					$theLogin = $routes['login'];

				}

				if((isset($routes['permissions']) 

					|| isset($routes[$this->method]['permissions'])) 

					&& ($routes[$this->method]['permissions'] ??

					 $routes['permissions']) && 

					!isset($routes['permissions'][$this->method])) {

					$thePermissions = $routes[$this->method]['permissions'] ?? $routes['permissions'];

				}

				if(isset($routes['redirect'])) {

					$theRedirect = $routes['redirect'];

				} else {

					$theRedirect = false;

				}

			}

		} else {

			http_response_code(404);

			header('location: ' . $this->config->routeRelative . $this->config->error404Route);

			return;

		}

		if ($theRedirect) {

			http_response_code(303);

			header('location: ' . $this->config->routeRelative . $theRedirect);

			return;

		} else

		if (isset($theLogin)) {

			if($theLogin && !$this->auth->isLoggedIn()) {

				http_response_code(401);

				$this->session->flash('error', 'Debes estar loggueado para poder entrar a esa página.');

				header('location: ' . $this->config->routeRelative . $this->config->loginFailRoute);

				return;

			}

			if(!$theLogin && $this->auth->isLoggedIn()) {

				header('location: ' . $this->config->routeRelative . $this->config->falseLoginRoute);

				return;

			}

		}

		if ($thePermissions && !$this->routes->checkPermission($thePermissions)) {

			http_response_code(403);

			header('location: ' . $this->config->routeRelative . $this->config->permissionFailRoute);

			return;

		}

		$controller = $routes[$this->method]['controller'];

		$action = $routes[$this->method]['action'];

		$page = $controller->$action();

		$json = $routes[$this->method]['json'] ?? false;
		if($json || (isset($page['jsonResponse']) && $page['jsonResponse'] == true)) {
			header('Content-Type: application/json');
			echo json_encode($page);
			return;
		}

		$title = $page['title'];

		$variables = $page['variables'] ?? [];

		$variables['auth'] = $this->auth->isLoggedIn();

		$this->helper = new \Dynamite\Helper($this->auth, $this->session, $this->config, $this->route);

		$output = $this->loadTemplate($page['template'], $variables);

		if(isset($page['layout'])){

			$layout = $page['layout'];

		}else{

			$layout = $this->config->defaultLayoutTemplate;

		}

		echo $this->loadTemplate($this->config->layoutRoute.$layout, [

			'output' => $output,

			'title' => $title

		]);

	}



	private function nestRoute(string $findRoute, array $routes, array $oldPath = []) {

		foreach ($routes as $key => $route) {

			if(is_array($route)) {

				$path = $oldPath;

				$path[] = $key;

				$text = '';

				foreach ($path as $pathValue) {

					$text .= $pathValue;

				}

				if($text == $findRoute) {

					return $path;

				} else if ($this->nestRoute($findRoute, $route, $path)){

					return $this->nestRoute($findRoute, $route, $path);

				}

			}

		}

	}



}



