<?php

namespace Routes;



use App\Controllers as Controller;



class Router implements \Dynamite\Routes {



	private $auth;

	private $tablaUsuarios;

	private $tablaLicenciaturas;

	private $tablaMaterias;

	private $tablaLicenciaturaMateria;

	private $tablaClases;

	private $tablaPE;

	private $tablaAlumnoClase;

	private $tablaPagos;
	private $tablaTitulacion;
	private $mensajes;



	public function __construct() {

		include __DIR__ . '/../includes/DatabaseConnection.php';



		$this->tablaUsuarios = new \Dynamite\DatabaseTable($pdo, 'usuarios', ['id'], '\App\Entity\Usuario', [

			&$this->tablaClases,

			&$this->tablaAlumnoClase,

			&$this->tablaLicenciaturas,
			&$this->mensajes
		]);

		$this->tablaLicenciaturas = new \Dynamite\DatabaseTable($pdo, 'licenciaturas', ['id'], '\App\Entity\Licenciatura', [

			&$this->tablaLicenciaturaMateria

		]);

		$this->tablaMaterias = new \Dynamite\DatabaseTable($pdo, 'materias', ['id'], '\App\Entity\Materia', [
			&$this->tablaLicenciaturaMateria
		]);

		$this->mensajes = new \Dynamite\DatabaseTable($pdo, 'mensajes', ['id']);

		$this->tablaTitulacion = new \Dynamite\DatabaseTable($pdo, 'titulacion', ['id_usuario']);

		$this->tablaLicenciaturaMateria = new \Dynamite\DatabaseTable($pdo, 'licenciaturas_materias', [

			'idLicenciatura' => 'idLicenciatura',

			'idMateria' => 'idMateria'

		], '\App\Entity\LicenciaturaMateria', [

			&$this->tablaLicenciaturas,

			&$this->tablaMaterias

		]);

		$this->tablaClases = new \Dynamite\DatabaseTable($pdo, 'clases', ['id'], '\App\Entity\Clase', [

			&$this->tablaMaterias,

			&$this->tablaUsuarios,

			&$this->tablaPE,

			&$this->tablaAlumnoClase

		]);

		$this->tablaPE = new \Dynamite\DatabaseTable($pdo, 'periodo_escolar', ['id'], '\App\Entity\PeriodoEscolar', [

			&$this->tablaClases

		]);

		$this->tablaAlumnoClase = new \Dynamite\DatabaseTable($pdo, 'alumno_clase', [

			'idAlumno' => 'idAlumno',

			'idClase' => 'idClase'

		], '\App\Entity\AlumnoClase', [

			&$this->tablaUsuarios,

			&$this->tablaClases

		]);

		$this->tablaPagos = new \Dynamite\DatabaseTable($pdo, 'pagos', ['id'], '\App\Entity\Pago', [

			&$this->tablaUsuarios

		]);

		$this->auth = new \Dynamite\Authentication($this->tablaUsuarios, 'correo', 'password');

	}



	public function getRoutes(): array {



		$app = new Controller\App($this->auth, $this->tablaUsuarios);

		$licenciaturas = new Controller\LicenciaturasController($this->tablaLicenciaturas, $this->tablaMaterias, $this->tablaLicenciaturaMateria);

		$materias = new Controller\MateriasController($this->tablaMaterias);

		$usuarios = new Controller\UsuarioController($this->tablaUsuarios, $this->tablaPagos, $this->tablaLicenciaturas, $this->tablaTitulacion, $this->mensajes, $this->auth);

		$periodos = new Controller\PEController($this->tablaPE);

		$clases = new Controller\ClasesController($this->auth, $this->tablaClases, $this->tablaMaterias, $this->tablaUsuarios, $this->tablaPE, $this->tablaAlumnoClase, $this->tablaLicenciaturas);

		$alumno = new Controller\AlumnoController($this->auth, $this->tablaAlumnoClase, $this->tablaUsuarios);

		$pagos = new Controller\Pagos($this->tablaPagos);

		$routes = [

			'' => [

				'GET' => [

					'controller' => $app,

					'action' => 'index'

				],

				'login' => false

			],

			'error/' => [

				'404' => [

					'GET' => [

						'controller' => $app,

						'action' => 'err404'

					]

				],

				'login' => false

			],

			'login' => [

				'POST' => [

					'controller' => $app,

					'action' => 'processLogin'

				],

				'login' => false

			],

			'logout' => [

				'POST' => [

					'controller' => $app,

					'action' => 'logout'

				],

				'login' => true

			],

			'panel/' => [

				'login' => true,

				'GET' => [

					'controller' => $app,

					'action' => 'dashboard'

				],

				'mis-clases/' => [

					'GET' => [

						'controller' => $alumno,

						'action' => 'misClases'

					],

					'permissions' => [\App\Entity\Usuario::ALUMNO]

				],

				'mensajes' => [
					'GET' => [
						'controller' => $usuarios,
						'action' => 'mensajeria'
					],
					'POST' => [
						'controller' => $usuarios,
						'action' => 'enviarMensaje'
					],
					'permissions' => [\App\Entity\Usuario::ADMINISTRADOR, \App\Entity\Usuario::DIRECTOR]
				],

				'mensajeria' => [
					'GET' => [
						'controller' => $usuarios,
						'action' => 'mensajesAlumno'
					]
				],

				'documentos' => [
					'GET' => [
						'controller' => $usuarios,
						'action' => 'documentos'
					],
					'POST' => [
						'controller' => $usuarios,
						'action' => 'subirDocumentos'
					],
					'permissions' => [\App\Entity\Usuario::ADMINISTRADOR, \App\Entity\Usuario::DIRECTOR, \App\Entity\Usuario::ALUMNO]
				],

				'servicio' => [
					'GET' => [
						'controller' => $usuarios,
						'action' => 'servicio'
					],
					'permissions' => [\App\Entity\Usuario::ADMINISTRADOR, \App\Entity\Usuario::DIRECTOR, \App\Entity\Usuario::ALUMNO]
				],

				'titulacion' => [
					'GET' => [
						'controller' => $usuarios,
						'action' => 'titulacion'
					],
					
				],

				'fechas-titulacion' => [
					'permissions' => [\App\Entity\Usuario::ADMINISTRADOR, \App\Entity\Usuario::DIRECTOR, \App\Entity\Usuario::ALUMNO],
						'GET' => [
							'controller' => $usuarios,
							'action' => 'titulacionData'
						],
						'POST' => [
							'controller' => $usuarios,
							'action' => 'cambiarFechas'
						]
				],

				'kardex' => [

					'GET' => [

						'controller' => $alumno,

						'action' => 'kardex'

					],

					'permissions' => [\App\Entity\Usuario::ADMINISTRADOR, \App\Entity\Usuario::DIRECTOR, \App\Entity\Usuario::ALUMNO]

				],

				'licenciaturas/' => [

					'GET' => [

						'controller' => $licenciaturas,

						'action' => 'index'

					],

					'crear' => [

						'GET' => [

							'controller' => $licenciaturas,

							'action' => 'create'

						],

						'POST' => [

							'controller' => $licenciaturas,

							'action' => 'store'

						]

					],

					'editar' => [

						'GET' => [

							'controller' => $licenciaturas,

							'action' => 'edit'

						],

						'POST' => [

							'controller' => $licenciaturas,

							'action' => 'update'

						]

					],

					'estado' => [

						'POST' => [

							'controller' => $licenciaturas,

							'action' => 'changeStatus'

						]

					],

					'asignar' => [

						'GET' => [

							'controller' => $licenciaturas,

							'action' => 'agregar'

						],

						'POST' => [

							'controller' => $licenciaturas,

							'action' => 'asignar'

						]

					],

					'eliminar-asignar' => [

						'POST' => [

							'controller' => $licenciaturas,

							'action' => 'eliminarAsignar'

						]

					],

					'permissions' => [\App\Entity\Usuario::ADMINISTRADOR, \App\Entity\Usuario::DIRECTOR]

				],

				'materias/' => [

					'GET' => [

						'controller' => $materias,

						'action' => 'index'

					],

					'crear' => [

						'GET' => [

							'controller' => $materias,

							'action' => 'create'

						],

						'POST' => [

							'controller' => $materias,

							'action' => 'store'

						]

					],

					'editar' => [

						'GET' => [

							'controller' => $materias,

							'action' => 'edit'

						],

						'POST' => [

							'controller' => $materias,

							'action' => 'update'

						]

					],

					'estado' => [

						'POST' => [

							'controller' => $materias,

							'action' => 'changeStatus'

						]

					],

					'permissions' => [\App\Entity\Usuario::ADMINISTRADOR, \App\Entity\Usuario::DIRECTOR]

				],

				'clases/' => [

					'GET' => [

						'controller' => $clases,

						'action' => 'index'

					],
					'POST' => [
						'controller' => $clases,
						'action' => 'getMaterias',
						'json' => true
					],

					'ver' => [

						'GET' => [

							'controller' => $clases,

							'action' => 'ver'

						]

					],

					'crear' => [

						'GET' => [

							'controller' => $clases,

							'action' => 'create'

						],

						'POST' => [

							'controller' => $clases,

							'action' => 'store'

						],

						'permissions' => [\App\Entity\Usuario::ADMINISTRADOR]

					],

					'editar' => [

						'GET' => [

							'controller' => $clases,

							'action' => 'edit'

						],

						'POST' => [

							'controller' => $clases,

							'action' => 'update'

						],

						'permissions' => [\App\Entity\Usuario::ADMINISTRADOR]

					],

					'estado' => [

						'POST' => [

							'controller' => $clases,

							'action' => 'changeStatus',

							'permissions' => [\App\Entity\Usuario::ADMINISTRADOR]

						]

					],

					'asignar' => [

						'GET' => [

							'controller' => $clases,

							'action' => 'agregar'

						],

						'POST' => [

							'controller' => $clases,

							'action' => 'asignar',

							'permissions' => [\App\Entity\Usuario::ADMINISTRADOR]

						]

					],

					'eliminar-asignar' => [

						'POST' => [

							'controller' => $clases,

							'action' => 'eliminarAsignar'

						],

						'permissions' => [\App\Entity\Usuario::ADMINISTRADOR]

					],

					'permissions' => [\App\Entity\Usuario::ADMINISTRADOR, \App\Entity\Usuario::PROFESOR]

				],

				'profesores/' => [

					'GET' => [

						'controller' => $usuarios,

						'action' => 'indexProfesor'

					],

					'crear' => [

						'GET' => [

							'controller' => $usuarios,

							'action' => 'createProfesor'

						],

						'POST' => [

							'controller' => $usuarios,

							'action' => 'storeProfesor'

						]

					],

					'editar' => [

						'GET' => [

							'controller' => $usuarios,

							'action' => 'editProfesor'

						],

						'POST' => [

							'controller' => $usuarios,

							'action' => 'updateProfesor'

						]

					],

					'estado' => [

						'POST' => [

							'controller' => $usuarios,

							'action' => 'changeStatusProfesor'

						]

					],

					'permissions' => [\App\Entity\Usuario::ADMINISTRADOR, \App\Entity\Usuario::DIRECTOR]

				],

				'pagos/' => [

					'GET' => [

						'controller' => $pagos,

						'action' => 'index'

					],

					'excel' => [

						'GET' => [

							'controller' => $usuarios,

							'action' => 'exportExcel'

						]

					]

				],	

				'alumnos/' => [

					'GET' => [

						'controller' => $usuarios,

						'action' => 'indexAlumno'

					],
					'crear' => [

						'GET' => [

							'controller' => $usuarios,

							'action' => 'createAlumno'

						],

						'POST' => [

							'controller' => $usuarios,

							'action' => 'storeAlumno'

						]

					],

					'editar' => [

						'GET' => [

							'controller' => $usuarios,

							'action' => 'editAlumno'

						],

						'POST' => [

							'controller' => $usuarios,

							'action' => 'updateAlumno'

						]

					],

					'estado' => [

						'POST' => [

							'controller' => $usuarios,

							'action' => 'changeStatusAlumno'

						]

					],

					'servicio' => [
						'POST' => [
							'controller' => $usuarios,
							'action' => 'changeServicio'
						]
					],

					'pagos' => [

						'GET' => [

							'controller' => $usuarios,

							'action' => 'pagos'

						],

						'POST' => [

							'controller' => $usuarios,

							'action' => 'subirPago'

						]

					],

					'excel' => [

						'GET' => [

							'controller' => $usuarios,

							'action' => 'exportExcel'

						]

					],

					'borrar-pago' => [

						'POST' => [

							'controller' => $usuarios,

							'action' => 'borrarPago'

						]

					],

					'permissions' => [\App\Entity\Usuario::ADMINISTRADOR]

				],

				'periodos/' => [

					'GET' => [

						'controller' => $periodos,

						'action' => 'index'

					],

					'crear' => [

						'GET' => [

							'controller' => $periodos,

							'action' => 'create'

						],

						'POST' => [

							'controller' => $periodos,

							'action' => 'store'

						]

					],

					'editar' => [

						'GET' => [

							'controller' => $periodos,

							'action' => 'edit'

						],

						'POST' => [

							'controller' => $periodos,

							'action' => 'update'

						]

					],

					'estado' => [

						'POST' => [

							'controller' => $periodos,

							'action' => 'changeStatus'

						]

					],

					'permissions' => [\App\Entity\Usuario::ADMINISTRADOR, \App\Entity\Usuario::DIRECTOR]

				],

				'calificaciones/' => [

					'GET' => [

						'controller' => $clases,

						'action' => 'indexCalificaciones'

					],

					'calificar' => [

						'GET' => [

							'controller' => $clases,

							'action' => 'calificarShow'

						],

						'POST' => [

							'controller' => $clases,

							'action' => 'calificar'

						]

					],

					'permissions' => [\App\Entity\Usuario::ADMINISTRADOR, \App\Entity\Usuario::PROFESOR]

				]

			]

		];

		return $routes;

	}



	public function getAuthentication(): \Dynamite\Authentication {

		return $this->auth;

	}



	public function checkPermission($tipoUsuarios): bool {

		$user = $this->auth->getUser();

		foreach ($tipoUsuarios as $tipoUsuario) {

			if($user && $user->esDelTipoUsuario($tipoUsuario)) {

				return true;

			}

		}

		return false;		

	}

}